@@ -36,6 +36,11 @@ import time
 from urllib import urlencode
 from urlparse import urljoin
 import webbrowser
+import ConfigParser
+from base64 import (
+    b64decode,
+    b64encode,
+    )
 
 import simplejson
 
@@ -69,6 +74,8 @@ class Credentials(OAuthAuthorizer):
 
     URI_TOKEN_FORMAT = "uri"
     DICT_TOKEN_FORMAT = "dict"
+    ITEM_SEPARATOR = '<BR>'
+    NEWLINE = '\n'
 
     def serialize(self):
         """Turn this object into a string.
@@ -77,7 +84,12 @@ class Credentials(OAuthAuthorizer):
         """
         sio = StringIO()
         self.save(sio)
-        return sio.getvalue()
+        serialized = sio.getvalue()
+        # Some users have reported problems with corrupted keyrings, both in
+        # Gnome and KDE, when newlines are included in the password.  Avoid
+        # this problem by base 64 encoding the serialized value.
+        serialized = b64encode(serialized)
+        return serialized
 
     @classmethod
     def from_string(cls, value):
@@ -86,6 +98,8 @@ class Credentials(OAuthAuthorizer):
         This should probably be moved into OAuthAuthorizer.
         """
         credentials = cls()
+        if 'consumer_key' not in value:
+            value = b64decode(value)
         credentials.load(StringIO(value))
         return credentials
 
@@ -121,7 +135,7 @@ class Credentials(OAuthAuthorizer):
             oauth_signature_method='PLAINTEXT',
             oauth_signature='&')
         url = web_root + request_token_page
-        headers = {'Referer' : web_root}
+        headers = {'Referer': web_root}
         if token_format == self.DICT_TOKEN_FORMAT:
             headers['Accept'] = 'application/json'
         response, content = httplib2.Http().request(
@@ -310,8 +324,9 @@ class KeyringCredentialStore(CredentialStore):
     def do_save(self, credentials, unique_key):
         """Store newly-authorized credentials in the keyring."""
         self._ensure_keyring_imported()
+        serialized = credentials.serialize()
         keyring.set_password(
-            'launchpadlib', unique_key, credentials.serialize())
+            'launchpadlib', unique_key, serialized)
 
     def do_load(self, unique_key):
         """Retrieve credentials from the keyring."""
@@ -319,7 +334,12 @@ class KeyringCredentialStore(CredentialStore):
         credential_string = keyring.get_password(
             'launchpadlib', unique_key)
         if credential_string is not None:
-            return Credentials.from_string(credential_string)
+            credential_string = credential_string.encode('utf8')
+            try:
+                credentials = Credentials.from_string(credential_string)
+                return credentials
+            except ConfigParser.NoOptionError:
+                return None
         return None
 
 
